/*
  TROLL (formerly ResHiggs)
  Example of usage for Drell-Yan
*/

#include <iomanip>
#include <vector>
#include <algorithm>
#include <iostream>
#include <fstream>
#include <string>
#include <sstream>
#include <stdio.h>
#include <cmath>
#include <ctime>
#include <cstdlib>
#include <sys/stat.h> 
#include <sys/time.h> 
#include <getopt.h> 

#include "./src/resummationQCD.hh"

using namespace std;



void print_usage() {
  cout << endl
       << "Usage:" << endl
       << " -p  --pdf-set <set>     specify PDFs being used (default \"NNPDF23_nnlo_as_0118\")" << endl
       << " -E  --Ecm <energy>      set collider energy [GeV]" << endl
       << " -m  --mass <mass>       set Drell-Yan pair invariant mass [GeV]" << endl
       << " -Y  --rapidity <rap>    set Drell-Yan pair rapidity Y (if not set, it resums the invariant mass distribution)" << endl
       << " -f  --fact <murat>      set muF/m ratio" << endl
       << " -r  --ren <murat>       set muR/m ratio" << endl
       << " -P  --prescription <p>  set resummation prescription: psi-soft (1), N-soft (2, default)" << endl
       << " -g  --g0exp             exponentiate constant function g0 or g0bar" << endl
       << endl;
  exit(0);
  return;
}




double muRrat = 1;
double muFrat = 1;
bool g0exp = false;
int prescr = 2;
bool doRapDistr = false;

const string sorder[4] = {"LO", "NLO", "NNLO", "NNNLO"};
void read_arguments(int argc, char* argv[], double &mH, double &sqrts, string &set, double &Y) {
  const char* const short_options = "hp:f:r:m:E:P:gY:";
  const struct option long_options[] = { { "help", 0, NULL, 'h' },
					 { "pdf-set", 1, NULL, 'p' },
					 { "fact", 1, NULL, 'f' },
					 { "ren", 1, NULL, 'r' },
					 { "mass", 1, NULL, 'm' },
					 { "Ecm", 1, NULL, 'E' },
					 { "prescription", 1, NULL, 'P' },
					 { "g0exp", 0, NULL, 'g' },
					 { "rapidity", 1, NULL, 'Y' },
					 { NULL, 0, NULL, 0 } };
  int next_option;
  ostringstream sset;
  do {
    next_option = getopt_long (argc, argv, short_options, long_options, NULL);
    switch (next_option) {
    case 'h':
      print_usage();
    case 'p':
      sset << optarg;
      set = sset.str();
      break;
    case 'f':
      muFrat = strtod(optarg, NULL);
      break;
    case 'r':
      muRrat = strtod(optarg, NULL);
      break;
    case 'm':
      mH = strtod(optarg, NULL);
      break;
    case 'E':
      sqrts = strtod(optarg, NULL);
      break;
    case 'P':
      prescr = strtol(optarg, NULL, 10);
      break;
    case 'g':
      g0exp = false;
      break;
    case 'Y':
      doRapDistr = true;
      Y = strtod(optarg, NULL);
    break;
    case '?':
      print_usage();
    case -1: break;
    default: abort();
    }
  }
  while (next_option != -1);
  //
  // messages
  //
  if(prescr==2 && g0exp==true)
    cout << endl << "\033[1m  WARNING:\033[0m "
	 << "\033[0;31m Using N-soft with g0 exponentiated: this is NOT the standard N-soft resummation prescription!\033[0m"
	 << endl << endl;
  //
  return;
}







int main (int argc, char* argv[]) {



  // default values (modified by runtime arguments)
  double Y = 0;
  double m = 100;
  double sqrts = 13000;
  string PDFset = "NNPDF30_nnlo_as_0118";


  // read runtime arguments
  read_arguments(argc, argv, m, sqrts, PDFset, Y);




  // Start time counter
  struct timeval t0, t1;
  gettimeofday(&t0,NULL);



  // *** Resummation part *** //
  //
  using namespace resumm;
  //dQCDResummation res(DYpositive, LHC, PDFset, 0);
  dQCDResummation res(DYneutral, LHC, PDFset, 0);
  //
  res.SetLogCounting(primed);
  //
  // Now we select the prescription
  // The easiest way uses the name of the prescription, that can be
  //    "A-soft_n"   or   "psi-soft_n"   or   "N-soft"
  // where n is the order of the Altarelli-Parisi improvement, and can be 0, 1 or 2
  string PName[] = { "A-soft_2", "psi-soft_2", "N-soft", "Psi-soft_2" };
  res.SetPrescriptionByName (PName[prescr], g0exp);
  res.SetMPpars(12);



  cout << "PDFset: " << PDFset << endl
       << "Renormalization scale muR = " << muRrat << " * m" << endl
       << "Factorization   scale muF = " << muFrat << " * m" << endl;
  if(doRapDistr) cout << "Y = " << Y << endl;
  cout << "m = " << m << " GeV" << endl
       << "prescription: " << PName[prescr] << endl
       << "exponentiate g0: " << (g0exp ? "yes" : "no") << endl
       << endl;
  //

  double xsNLL, xsNNLL, xsNNNLL;
  
  if(doRapDistr) {
    xsNLL   = res.xsRapNLL  (m, Y, sqrts, muFrat, muRrat, NLO);
    xsNNLL  = res.xsRapNNLL (m, Y, sqrts, muFrat, muRrat, NNLO);
    xsNNNLL = res.xsRapNNNLL(m, Y, sqrts, muFrat, muRrat, NNNLO);
  }
  else {
    xsNLL   = res.xsNLL  (m, sqrts, muFrat, muRrat, NLO);
    xsNNLL  = res.xsNNLL (m, sqrts, muFrat, muRrat, NNLO);
    xsNNNLL = res.xsNNNLL(m, sqrts, muFrat, muRrat, NNNLO);
  }

  cout << "Use this ouput to construct a resummed cross section as" << endl
       << "  sigma(NjLO+NjLL) = sigma(NjLO) + sigma(LO) * Delta_j K(NjLL)" << endl;
  cout << endl;

  cout << "Delta_1 K(NLL)   = " << xsNLL << endl
       << "Delta_2 K(NNLL)  = " << xsNNLL << endl
       << "Delta_3 K(NNNLL) = " << xsNNNLL << endl
       << endl;



  // Stop time counter
  gettimeofday(&t1,NULL);
  double t=t1.tv_sec-t0.tv_sec+(t1.tv_usec-t0.tv_usec)*0.000001;
  cout << endl << "Total time: " << t << "s" << endl;


  return 0;

}
