/*
  TROLL (formerly ResHiggs)
  Example of usage with ggHiggs for SCET resummation
*/

#include <iomanip>
#include <vector>
#include <algorithm>
#include <iostream>
#include <fstream>
#include <string>
#include <sstream>
#include <stdio.h>
#include <cmath>
#include <ctime>
#include <cstdlib>
#include <sys/stat.h> 
#include <sys/time.h> 
#include <getopt.h> 

#include "../ggHiggs/src/ggHiggs.hh"
#include "./src/resummationSCET.hh"

using namespace std;
using namespace ggHiggs;


/*
  If you like, you can customize your alpha_s routine.
  However, consistency with PDF sets suggests the use of the LHAPDF alphas routine.
*/
double ggHiggs::alphas(double mu) {
  return alphasPDF(mu);
  //return alphas_ihixs(mu, order, LHAPDF::alphasPDF(mZ), mZ); // taken from ihixs code
}





void print_usage() {
  cout << endl
       << "Usage:" << endl
       << " -p  --pdf-set <set>     specify PDF set" << endl
       << " -s  --PDFmember <mem>   specify PDF member of the set (default 0)" << endl
       << " -m  --mass <mass>       set Higgs mass [GeV]" << endl
       << " -E  --Ecm <energy>      set collider energy [GeV]" << endl
       << " -f  --fact <murat>      set muF/mH ratio" << endl
       << " -r  --ren <murat>       set muR/mH ratio" << endl
       << " -H  --hard <murat>      set muH/mH  ratio" << endl
       << " -t  --topmass <mass>    set top quark mass [GeV]" << endl
       << " -b  --include-bottom    include the bottom Yukawa coupling" << endl
       << " -c  --include-charm     include the charm  Yukawa coupling" << endl
       << " -e  --noEFT             use finite quark masses (default: infinite-mtop EFT approximation)" << endl
       << " -C  --collinear         include collinear contributions" << endl
       << " -2  --pi2               perform pi^2 resummation" << endl
       << endl;
  exit(0);
  return;
}




double muHrat = 1;
double muRrat = 1;
double muFrat = 1;
bool pi2 = false;
bool collinear = false;

const string sorder[4] = {"LO", "NLO", "NNLO", "NNNLO"};
void read_arguments(int argc, char* argv[], double &mH, double &sqrts, string &set, bool &nob, bool &noc) {
  const char* const short_options = "ht:ep:f:r:H:m:E:bcC2";
  const struct option long_options[] = { { "help", 0, NULL, 'h' },
					 { "topmass", 1, NULL, 't' },
					 { "noEFT", 0, NULL, 'e' },
					 { "pdf-set", 1, NULL, 'p' },
					 { "fact", 1, NULL, 'f' },
					 { "ren", 1, NULL, 'r' },
					 { "hard", 1, NULL, 'H' },
					 { "mass", 1, NULL, 'm' },
					 { "Ecm", 1, NULL, 'E' },
					 { "include-bottom", 0, NULL, 'b' },
					 { "include-charm", 0, NULL, 'c' },
					 { "collinear", 0, NULL, 'C' },
					 { "pi2", 0, NULL, '2' },
					 { NULL, 0, NULL, 0 } };
  int next_option;
  ostringstream sset;
  do {
    next_option = getopt_long (argc, argv, short_options, long_options, NULL);
    switch (next_option) {
    case 'h':
      print_usage();
    case 'e':
      finite_mt = true;
      break;
    case 't':
      mtop = strtod(optarg, NULL);
      break;
    case 'p':
      sset << optarg;
      set = sset.str();
      break;
    case 'f':
      muFrat = strtod(optarg, NULL);
      break;
    case 'r':
      muRrat = strtod(optarg, NULL);
      break;
    case 'H':
      muHrat = strtod(optarg, NULL);
      break;
    case 'm':
      mH = strtod(optarg, NULL);
      break;
    case 'E':
      sqrts = strtod(optarg, NULL);
      break;
    case 'C':
      collinear = true;
      break;
    case '2':
      pi2 = true;
      break;
    case 'b':
      nob = false;
      break;
    case 'c':
      noc = false;
      break;
    case '?':
      print_usage();
    case -1: break;
    default: abort();
    }
  }
  while (next_option != -1);
  //
  // messages
  //
  _muF_mu_ratio  = muFrat;
  _muR_muF_ratio = muRrat/muFrat;
  return;
}







int main (int argc, char* argv[]) {


  /*************  input parameters  **************/

  // pole masses (GeV)
  mtop = 172.5;

  // relative precision for integrals
  INTEGRAL_PRECISION = 5.e-3;

  // scheme at small x (set to "false" for Q0MSbar scheme)
  MSbar = true;

  /***********************************************/



  // default values (modified by runtime arguments)
  double mH = 125;
  double sqrts = 13000;
  string PDFset = "PDF4LHC15_nnlo_100";


  // read runtime arguments
  bool nob=true, noc=true; // when true, bottom and charm do NOT run into the loop coupling to the Higgs
  read_arguments(argc, argv, mH, sqrts, PDFset, nob, noc);




  // Start time counter
  struct timeval t0, t1;
  gettimeofday(&t0,NULL);


  // set to false to show intermediate K-factor results
  quiet = true;

  // init PDFs
  cout << "Loading PDF set " << PDFset << endl;
  int subset = 0;
  pdf_init(PDFset, subset);

  // masses (negative mass means the quark will not contribute to the loop)
  mbot = (nob ? -1 : 1) * getQuarkMass(5);
  mcha = (noc ? -1 : 1) * getQuarkMass(4);

  // init
  double tau = mH*mH/sqrts/sqrts;
  sxc = new SmallxCoeffs(mH,mtop,mbot,mcha);
  init_consts(mH);
  //SetScaleDependence(1,1);
  //
  cout << "Higgs mass:       " << mH << " GeV" << endl
       << "Collider energy:  " << sqrts << " GeV" << endl
       << "tau = mH^2/s =    " << tau << endl
       << "top    mass =     " << mtop << "  running in the loop" << endl
       << "bottom mass =     " << fabs(mbot) << "  " << (nob ? "NOT " : "") << "running in the loop" << endl
       << "charm  mass =     " << fabs(mcha) << "  " << (noc ? "NOT " : "") << "running in the loop" << endl
       << "quark mass effects beyond LO: " << (finite_mt ? "YES (exact theory)" : "NO (EFT)") << endl
       << "Hard scale muH =            " << muHrat << " * mH" << endl
       << "Renormalization scale muR = " << muRrat << " * mH" << endl
       << "Factorization   scale muF = " << muFrat << " * mH" << endl;
  //





  /******* RUN EXAMPLE *******/

  // Note also that the same PDF set has to be used at every order.
  double xsLO=1, xsNLO=0, xsNNLO=0,xsNNNLO=0;
  //
  double LOpref = xs_LO_prefactor(mH);
  double muF = mH * muFrat;
  double muR = mH * muRrat;
  double alpha_s = alphas(muR);
  // results
  double rexact[3];

  bool ggonly = false;

  // LO
  xsLO = tau*Lum(tau,muF)*LOpref*pow(alpha_s,2);

  // NLO
  order = 1;
  __all_channels__ = true & !ggonly;
  compute(mH, sqrts, xsLO, 0, 0, &rexact[0]);
  xsNLO = xsLO + rexact[0];

  // NNLO
  order = 2;
  __all_channels__ = true & !ggonly;
  compute(mH, sqrts, xsLO, xsNLO, 0, &rexact[1]);
  xsNNLO = xsNLO + rexact[1];

  // NNNLO
  order = 3;
  __all_channels__ = true & !ggonly;
  compute(mH, sqrts, xsLO, xsNLO, xsNNLO, &rexact[2]);
  xsNNNLO = xsNNLO + rexact[2];

  cout << endl << "Fixed order part: computed." << endl;


  // *** Resummation part *** //
  //
  using namespace resumm;
  SCETResummation res(ggH, LHC, PDFset, subset);
  //
  res.SetMtop(mtop);
  res.SetMbottom(mbot);
  res.SetMcharm(mcha);
  res.UseLargeMtopEFT(!finite_mt);
  res.SetLogCounting(primed);
  if(collinear) res.SetAPimprovement(AP1);
  res.UsePi2Resummation(pi2);
  //

  // you can specify the value of muS yourself
  //res.Setmus(...);
  // you might want to use res.GetmusI(mH,sqrts) and res.GetmusII(mH,sqrts), for computing scale uncertainty for instance.
  // If you want to restore the default, you can use res.Resetmus()


  // compute resummed contribution


  // compute resummed contribution
  
  //you can specify whether to include higher-order interferences or not
  //res.SetStrictExpansion(true);
  
  // compute resummed contribution
  double xsNLLp    = xsLO * res.xsNLL  (mH, sqrts, muFrat, muRrat,   NLO);
  double xsNNLLp   = xsLO * res.xsNNLL (mH, sqrts, muFrat, muRrat,  NNLO);
  double xsNNNLLp2 = xsLO * res.xsNNNLL(mH, sqrts, muFrat, muRrat,  NNLO);
  double xsNNNLLp3 = xsLO * res.xsNNNLL(mH, sqrts, muFrat, muRrat, NNNLO);
  //
  res.SetLogCounting(unprimed);
  double xsNLL   = xsLO * res.xsNLL  (mH, sqrts, muFrat, muRrat,   LO);
  double xsNNLL  = xsLO * res.xsNNLL (mH, sqrts, muFrat, muRrat,  NLO);
  double xsNNNLL = xsLO * res.xsNNNLL(mH, sqrts, muFrat, muRrat, NNLO);
  // end resummation part //


  cout << endl
       << "The following cross-sections (in pb) have been computed with the same PDF set (" << PDFset << ") at each order."
       << endl << endl;
  cout << "LO:         " << xsLO << endl;
  cout << "NLO:        " << xsNLO << endl;
  cout << "NNLO:       " << xsNNLO << endl;
  cout << "N3LO:       " << xsNNNLO << endl;
  cout << "LO+NLL:     " << xsLO+xsNLL << endl;
  cout << "NLO+NLL':   " << xsNLO+xsNLLp << endl;
  cout << "NLO+NNLL:   " << xsNLO+xsNNLL << endl;
  cout << "NNLO+NNLL': " << xsNNLO+xsNNLLp << endl;
  cout << "NNLO+N3LL:  " << xsNNLO+xsNNNLL << endl;
  cout << "NNLO+N3LL': " << xsNNLO+xsNNNLLp2 << endl;
  cout << "N3LO+N3LL': " << xsNNNLO+xsNNNLLp3 << endl;
  //





  // Stop time counter
  gettimeofday(&t1,NULL);
  double t=t1.tv_sec-t0.tv_sec+(t1.tv_usec-t0.tv_usec)*0.000001;
  cout << endl << "Total time: " << t << "s" << endl;


  return 0;

}
