#ifndef __resummation__
#define __resummation__

#include <string>
#include "luminosity.hh"
#include "PDFinterface.hh"
#include "math/complex.hh"

using namespace std;






namespace resumm {

  // actual alphas routine will be provided by the user
  //extern double alphasUser(double mu);


  enum log_order{
    LL    = 0,
    NLL   = 1,
    NNLL  = 2,
    NNNLL = 3
  };
  enum order{
    LO    = 0,
    NLO   = 1,
    NNLO  = 2,
    NNNLO = 3
  };
  enum prescription {
    BP00 =  10,  // resums (log^k log(1/z) / log(1/z))_+
    BP0  =  11,  // resums (log^k log(1/z) / log(1/z))_+ /sqrt(z)
    BP1  =  1,   // resums (log^k (1-z) / (1-z))_+
    BP2  =  2,   // resums (log^k ((1-z)/sqrt(z)^a) / (1-z))_+    (a=1 for Higgs/DY, a=2 for DIS)
    //
    MP00 = -10,  // same logs as BP00 (equivalent in practice): log(N)
    MP0  = -11,  // same logs as BP0  (equivalent in practice): log(N-1/2)
    MP2  = -2,   // using psi0(N) instead of log(N-1/2) (formally almost equivalent to BP2, and similar to MP0=BP0)
    MP5  = -5    // using 2*psi0(N)-3*psi(N+1)+2*psi(N+2) (with AP0 reproduces the results of 1602.00695)
  };
  enum APtype{
    AP0  = 0,  // AP to order 0
    AP1  = 1,  // AP to order 1
    AP2  = 2,  // AP to order 2
    AP   = -2  // full AP
  };
  enum LogCountType{
    primed = 0,
    unstarred = 0,
    appropriateforC = 0,
    Catani = 0,
    //
    unprimed = 1,
    starred = 1,
    appropriateforlogC = 1,
    Neubert = 1
  };

  extern string sPrescr(prescription p);
  extern string sAP(APtype ap);


  class Resummation {

  public:
    Resummation(process p, collider c, string PDFset, int PDFmember=0);
    //Resummation(const Resummation &r);
    ~Resummation();

    // re-initialize PDFs (use it only if you need to change PDF set; however, a better practice is to use another instance of Resummation)
    void reinitPDF(string PDFset, int PDFmember=0);

    void InitMasses        (double mc,  double mb,  double mt );
    void InitMassThresholds(double muc, double mub, double mut);
    void Setmuf(double mu); // I should delete this (used just for luminosity tests)
    void SetMtop   (double mt); // value used in Higgs resummation (in principle not the same as top threshold)
    void SetMbottom(double mb);
    void SetMcharm (double mc);
    void UseLargeMtopEFT(bool use=true); // set to false for the exact result
    void SetTargetMass(double target_mass);  // has an effect only for DIS; in GeV
    PDFinterface::PDFs *GetPDFs();
    double alphas(double mu);

    // select prescription (which mainly means "select subleading terms")
    // last argument is effective only with Borel summation
    void SetPrescriptionByName(string pr, bool exponentiate_g0);
    void SetPrescription(prescription p=BP2, bool exponentiate_g0=true, bool resumDistrConst=false, bool expBorelPrescription=false);
    void SetAPimprovement(APtype impr=AP1);
    void SetBPpars(unsigned int Ncheb=10, double cutoff=2.);
    void SetMPpars(unsigned int Ncheb=10);
    void SetMPpars(unsigned int Ncheb, double beta, int gamma, double z_min=1.e-5);
    void SetInterpolationMethod(string name); // Chebyshev, Lagrange
    void SetDamping(unsigned int strenght=3); // 0: no damping (default), 1: strongest damping (discouraged), 2: strong damping, 3+:milder dampings
    void SetConstSubtraction(dcomplex N); // subtract the Delta_resummed Coefficient Function at N
    void UnsetConstSubtraction();
    void SetLogCounting(LogCountType lct = primed);
    void ComputeCommon(bool redo);

    // for DIS
    double DISxsLL   (double x, double Q2, double y=0, double muF_m_ratio=1., double muR_m_ratio=1., order match_order=LO   , double *asinput=NULL);
    double DISxsNLL  (double x, double Q2, double y=0, double muF_m_ratio=1., double muR_m_ratio=1., order match_order=NLO  , double *asinput=NULL);
    double DISxsNNLL (double x, double Q2, double y=0, double muF_m_ratio=1., double muR_m_ratio=1., order match_order=NNLO , double *asinput=NULL);
    double DISxsNNNLL(double x, double Q2, double y=0, double muF_m_ratio=1., double muR_m_ratio=1., order match_order=NNNLO, double *asinput=NULL);

    // for Higgs and Drell-Yan inclusive cross section (invariant mass distribution)
    double xsLL   (double m, double sqrt_s, double muF_m_ratio=1., double muR_m_ratio=1., order match_order=LO  , double *asinput=NULL);
    double xsNLL  (double m, double sqrt_s, double muF_m_ratio=1., double muR_m_ratio=1., order match_order=NLO , double *asinput=NULL);
    double xsNNLL (double m, double sqrt_s, double muF_m_ratio=1., double muR_m_ratio=1., order match_order=NNLO, double *asinput=NULL);
    double xsNNNLL(double m, double sqrt_s, double muF_m_ratio=1., double muR_m_ratio=1., order match_order=NNLO, double *asinput=NULL);
    virtual double res_xs (double m, double sqrt_s, double muF_m_ratio, double muR_m_ratio, log_order LOGo, order match_order, double *asinput=NULL) =0;

    // for Higgs and Drell-Yan double-differential invariant mass and rapidity distribution
    double xsRapLL   (double m, double Y, double sqrt_s, double muF_m_ratio=1., double muR_m_ratio=1., order match_order=LO  , double *asinput=NULL);
    double xsRapNLL  (double m, double Y, double sqrt_s, double muF_m_ratio=1., double muR_m_ratio=1., order match_order=NLO , double *asinput=NULL);
    double xsRapNNLL (double m, double Y, double sqrt_s, double muF_m_ratio=1., double muR_m_ratio=1., order match_order=NNLO, double *asinput=NULL);
    double xsRapNNNLL(double m, double Y, double sqrt_s, double muF_m_ratio=1., double muR_m_ratio=1., order match_order=NNLO, double *asinput=NULL);
    virtual double res_xs_rap(double m, double Y, double sqrt_s, double muF_m_ratio, double muR_m_ratio, log_order LOGo, order match_order, double *asinput=NULL) =0;


  protected:
    double mu_c,   mu_b,   mu_t;   // quark thresholds
    double mass_c, mass_b, mass_t; // quark masses
    //
    process  proc;
    collider coll;
    bool doRapidityDistr;
    prescription prescr;
    APtype APimpr;
    bool g0exp;
    bool expDconst;
    bool expBP;
    //
    PDFinterface::PDFs *pdfs;
    //
    int interpolationMethod;
    unsigned int Nchebyshev;
    double *coeff;
    //
    double BPcutoff;
    double MPbeta, MPzmin;
    int MPgamma;
    double MP_reN0, MP_slope;
    //
    log_order LL_order;
    LogCountType LogarithmicCounting;
    order matching_order;
    unsigned int DampingOrder;
    dcomplex Nsubtraction;
    bool doNsubtraction;
    //
    double tau, luminosity_in_tau;
    double rapY;
    double DISy, TargetMass;
    double M, muF, muR;
    double Hmtop, Hmbot, Hmcha; // masses used for Higgs
    int nf;
    int nf_of_mu(double mu);
    double as, ab, asrescale;
    double b0, b1, b2, b3;
    double A1, A2, A3, A4, D1, D2, D3;
    double EG, Z2, Z3;
    double d1c, d2c, d3c;
    double gA0, gA1, gB0, gB1, gB2;
    double g0();
    double g01, g02, g03;
    //double g01, g02, g03;
    //double d1, d2, d3;
    bool g0constComputation;
    double g0sub1, g0sub2, g0sub3;
    double g0sub1s, g0sub2s, g0sub3s;
    double g0bar1, g0bar2, g0bar3;
    double W1, W2, W3;
    bool mtEFT;
    //
    double Nlandau();
    friend double LumRescaled(double x, void *p);
    friend double Lum(double x, void *p);
    //
    dcomplex mellD(int, dcomplex);
    dcomplex APimprovement(dcomplex (Resummation::*f)(dcomplex,void*), dcomplex N, void *p=NULL);
    //double  g0();
    double  g0sub();
    dcomplex g1(dcomplex);
    dcomplex g2(dcomplex);
    dcomplex g3(dcomplex);
    dcomplex g4(dcomplex);
    dcomplex S(dcomplex, void*);
    dcomplex asS1 (dcomplex,void*);
    dcomplex as2S2(dcomplex,void*);
    dcomplex as3S3(dcomplex,void*);
    double asg01 ();
    double as2g02();
    double as3g03();
    dcomplex g0expS_order1(dcomplex);
    dcomplex g0expS_order2(dcomplex);
    dcomplex g0expS_order3(dcomplex);
    dcomplex expS(dcomplex);
    dcomplex g0expS_matched_predamping(dcomplex);
    dcomplex g0expS_matched_presubtraction(dcomplex);
    dcomplex g0expS_matched(dcomplex);
    //
    dcomplex NofZ(dcomplex z, double ab);
    dcomplex ZofN(dcomplex N, double ab);
    dcomplex NofN(dcomplex N);
    //
    void InitResummCoeffs();
    void InitCusp(bool adjoint);
    void InitCoeffDY();
    void InitConstDY();
    void InitCoeffHiggs();
    void InitConstHiggs(bool pseudoscalar);
    void InitConstHiggsCommons(bool pseudoscalar);
    void InitCoeffDIS();
    void InitConstDIS();
    //
    dcomplex delta_term(dcomplex);
    dcomplex cheb_term(dcomplex, int);
    dcomplex convolution(dcomplex);
    dcomplex BorelIntegrand(double, dcomplex);
    double  BorelIntegrand_chvar(double, double);
    friend int integrandCUBA(const int *ndim, const double x[], const int *ncomp, double res[], void *pars);
    double BorelPrescription();
    //
    dcomplex NLogGen(dcomplex, void*);
    dcomplex BorelIntegrandExp(double, dcomplex, dcomplex);
    dcomplex BorelIntegrandExp_chvar(double, double, dcomplex);
    friend int integrandCUBAexp(const int *ndim, const double x[], const int *ncomp, double res[], void *pars);
    dcomplex BorelPrescriptionExp(dcomplex);
    //
    dcomplex LuminosityN(dcomplex N);
    dcomplex lum_times_g0expS_matched(dcomplex N);
    friend double Minimal_integrand(double t, void *p);
    friend double Minimal_integrand_inf(double t, void *p);
    dcomplex InverseMellinIntegrand(dcomplex N);
    double InverseMellin();
    double MinimalPrescription();
    //
    // cache
    double last_muFrat;
    double last_muRrat;
    double last_M;
    double last_Y;
    double last_sqrts;
    double last_mtop;
    float last_nf;
    float last_Ncheb;
    float last_EFT;
    float firstinit;
    bool forceredo;

  };



};




#endif
